"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var fetchPonyfill = require("fetch-ponyfill");
var qs = require("qs");
var query_1 = require("./query");
var SEARCH = '/search';
var REFINEMENTS = '/refinements';
var INVALID_QUERY_ERROR = 'query was not of a recognised type';
var createTimeoutPromise = function (timeout) {
    return new Promise(function (resolve, reject) {
        setTimeout(function () {
            reject(new BridgeTimeout("Timed out in " + timeout + " ms"));
        }, timeout);
    });
};
var BridgeTimeout = (function (_super) {
    __extends(BridgeTimeout, _super);
    /* istanbul ignore next */
    function BridgeTimeout(err) {
        return _super.call(this, err) || this;
    }
    return BridgeTimeout;
}(Error));
exports.BridgeTimeout = BridgeTimeout;
var BridgeError = (function (_super) {
    __extends(BridgeError, _super);
    /* istanbul ignore next */
    function BridgeError(statusText, status, data) {
        var _this = _super.call(this, statusText) || this;
        _this.status = status;
        _this.data = data;
        Object.setPrototypeOf(_this, BridgeError.prototype);
        return _this;
    }
    Object.defineProperty(BridgeError.prototype, "statusText", {
        get: function () {
            return this.message;
        },
        enumerable: true,
        configurable: true
    });
    return BridgeError;
}(Error));
exports.BridgeError = BridgeError;
exports.DEFAULT_CONFIG = {
    timeout: 1500
};
var AbstractBridge = (function () {
    function AbstractBridge(config) {
        this.fetch = fetchPonyfill().fetch;
        this.headers = {};
        this.config = Object.assign({}, exports.DEFAULT_CONFIG, config);
    }
    AbstractBridge.prototype.search = function (query, callback) {
        var _a = this.extractRequest(query), request = _a.request, queryParams = _a.queryParams;
        if (request === null)
            return this.generateError(INVALID_QUERY_ERROR, callback);
        var response = this.fireRequest(this.bridgeUrl, request, queryParams)
            .then(AbstractBridge.transformRecords)
            .then(AbstractBridge.transformRefinements);
        return this.handleResponse(response, callback);
    };
    AbstractBridge.prototype.refinements = function (query, navigationName, callback) {
        var request = this.extractRequest(query).request;
        if (request === null)
            return this.generateError(INVALID_QUERY_ERROR, callback);
        var refinementsRequest = { originalQuery: request, navigationName: navigationName };
        var response = this.fireRequest(this.refinementsUrl, refinementsRequest);
        return this.handleResponse(response, callback);
    };
    AbstractBridge.prototype.handleResponse = function (response, callback) {
        if (callback) {
            response.then(function (res) { return callback(undefined, res); }, function (err) { return callback(err); });
        }
        else {
            return response;
        }
    };
    AbstractBridge.prototype.extractRequest = function (query) {
        switch (typeof query) {
            case 'string': return { request: new query_1.Query(query).build(), queryParams: {} };
            case 'object': return query instanceof query_1.Query
                ? { request: query.build(), queryParams: query.queryParams }
                : { request: query, queryParams: {} };
            default: return { request: null, queryParams: null };
        }
    };
    AbstractBridge.prototype.generateError = function (error, callback) {
        var err = new Error(error);
        if (callback) {
            callback(err);
        }
        else {
            return Promise.reject(err);
        }
    };
    // tslint:disable-next-line max-line-length
    AbstractBridge.prototype.fireRequest = function (url, body, queryParams) {
        var _this = this;
        if (queryParams === void 0) { queryParams = {}; }
        var options = {
            method: 'POST',
            headers: __assign({ 'Content-Type': 'application/json' }, this.headers),
            body: JSON.stringify(this.augmentRequest(body)),
        };
        var params = qs.stringify(queryParams);
        url = params ? url + "?" + params : url;
        return Promise.race([this.fetch(url, options), createTimeoutPromise(this.config.timeout)])
            .then(function (res) {
            if (res.ok) {
                return res.json();
            }
            else {
                return res.json().then(function (err) {
                    throw new BridgeError(res.statusText, res.status, err);
                });
            }
        })
            .catch(function (err) {
            if (_this.errorHandler) {
                _this.errorHandler(err);
            }
            throw err;
        });
    };
    AbstractBridge.transform = function (response, key, callback) {
        if (response[key]) {
            return Object.assign(response, (_a = {}, _a[key] = response[key].map(callback), _a));
        }
        else {
            return response;
        }
        var _a;
    };
    AbstractBridge.transformRecords = function (response) {
        return AbstractBridge.transform(response, 'records', AbstractBridge.convertRecordFields);
    };
    AbstractBridge.transformRefinements = function (response) {
        var transformed = AbstractBridge.transform(response, 'availableNavigation', AbstractBridge.convertRefinement);
        return AbstractBridge.transform(transformed, 'selectedNavigation', AbstractBridge.convertRefinement);
    };
    AbstractBridge.convertRecordFields = function (record) {
        var converted = Object.assign(record, { id: record._id, url: record._u, title: record._t });
        delete converted._id;
        delete converted._u;
        delete converted._t;
        if (record._snippet) {
            converted.snippet = record._snippet;
            delete converted._snippet;
        }
        return converted;
    };
    AbstractBridge.convertRefinement = function (navigation) {
        if (navigation.range) {
            navigation.min = Number.MAX_SAFE_INTEGER;
            navigation.max = Number.MIN_SAFE_INTEGER;
            navigation.refinements = navigation.refinements
                .map(function (ref) {
                navigation.min = Math.min(navigation.min, ref.low);
                navigation.max = Math.max(navigation.max, ref.high);
                return (__assign({}, ref, { high: parseFloat(ref.high), low: parseFloat(ref.low) }));
            });
        }
        return navigation;
    };
    return AbstractBridge;
}());
exports.AbstractBridge = AbstractBridge;
var CloudBridge = (function (_super) {
    __extends(CloudBridge, _super);
    function CloudBridge(clientKey, customerId, config) {
        if (config === void 0) { config = {}; }
        var _this = _super.call(this, config) || this;
        _this.clientKey = clientKey;
        _this.baseUrl = "https://" + customerId + ".groupbycloud.com:443/api/v1";
        _this.bridgeUrl = _this.baseUrl + SEARCH;
        _this.refinementsUrl = _this.bridgeUrl + REFINEMENTS;
        return _this;
    }
    CloudBridge.prototype.augmentRequest = function (request) {
        return Object.assign(request, { clientKey: this.clientKey });
    };
    return CloudBridge;
}(AbstractBridge));
exports.CloudBridge = CloudBridge;
var BrowserBridge = (function (_super) {
    __extends(BrowserBridge, _super);
    function BrowserBridge(customerId, https, config) {
        if (https === void 0) { https = false; }
        if (config === void 0) { config = {}; }
        var _this = _super.call(this, config) || this;
        var scheme = https ? 'https' : 'http';
        var port = https ? ':443' : '';
        _this.baseUrl = scheme + "://" + customerId + "-cors.groupbycloud.com" + port + "/api/v1";
        _this.bridgeUrl = _this.baseUrl + SEARCH;
        _this.refinementsUrl = _this.bridgeUrl + REFINEMENTS;
        return _this;
    }
    BrowserBridge.prototype.augmentRequest = function (request) {
        return request;
    };
    return BrowserBridge;
}(AbstractBridge));
exports.BrowserBridge = BrowserBridge;
//# sourceMappingURL=bridge.js.map
//# sourceMappingURL=bridge.js.map